#!/bin/bash

# Copyright (C) 2023, Xilinx Inc - All rights reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"). You may
# not use this file except in compliance with the License. A copy of the
# License is located at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

echo "Number of args $#"

helper_func()
{
   echo ""
   echo "Usage:-"
   echo "    $0"
   echo "        or"
   echo "    $0 \"Absolute path of ma35/build folder\" \"Absolute path of ma35_apps folder\" \"Absolute path of Input_sources folder\""
}

# override if all required paths are specified as an arguments
if [ $# -gt 3 ] || ([ $# -gt 0 ] && [ $# -ne 3 ]) 
then
    helper_func
    exit 1
fi

# modify these parameters as per your test requirements.
# right now, these are set as per the test cases defined in "ma35_apps/xrm_apps/doc/ma35_xrm_test_cases.xlsx"
# configure number of test cases
NUM_TESTCASES=22
# configure number of input source files for each test case
NUM_SOURCES=10
# controls the inputs to each testcase. multi-input test cases need to be handled separately.
# Below inputs need to be available in INPUT_SOURCES_PATH defined below.
INPUTS=(4kp_dummy.264 4kp60.264 4kp60.yuv 4kp60.yuv 4kp60.yuv 4kp60.264 4kp60.264 4kp60.yuv 4kp60.264 4kp60.264 4kp60.264 4kp60.264 4kp60.264 4kp60.264 4kp60.yuv 4kp60.yuv 4kp60.264 4kp60.264 4kp60.264 4kp60.hevc 4kp60.264 4kp60.hevc 4kp60.hevc)

# modify below paths as per your test requirements
echo ""
echo "##### Modify below paths by sending(absolute paths) as an argument in the same order #####"
MA35_BUILD_PATH=$PWD
MA35_APPS_PATH=$PWD/_deps/apps-src/
INPUT_SOURCES_PATH=/home/mapped/git/misc/inputs
JSON_PATH=$MA35_APPS_PATH/xrm_apps/test_scripts/describe_job

if [ $# -gt 0 ]
then
    MA35_BUILD_PATH=$1
	MA35_APPS_PATH=$2
	INPUT_SOURCES_PATH=$3
	JSON_PATH=$MA35_APPS_PATH/xrm_apps/test_scripts/describe_job
fi

echo "MA35_BUILD_PATH: " $MA35_BUILD_PATH
echo "MA35_APPS_PATH: " $MA35_APPS_PATH
echo "INPUT_SOURCES_PATH: " $INPUT_SOURCES_PATH
echo "#########################################################################"
echo ""

# derive remaining paths and create directories
RUN_CMDS_PATH=$JSON_PATH/run_cmds.txt
JOBSLOT_EXE_PATH=$MA35_BUILD_PATH/_deps/apps-build/xrm_apps/jobslot_reservation/jobslot_reservation
LAUNCHER_EXE_PATH=$MA35_BUILD_PATH/_deps/apps-build/xrm_apps/launcher/launcher
JOBSLOT_TEST_DIR="$MA35_BUILD_PATH/jobslot_reservation"
LAUNCHER_TEST_DIR="$MA35_BUILD_PATH/launcher"
LAUNCHER_RUNCMD_PATH=$LAUNCHER_TEST_DIR/run_params
LAUNCHER_SOURCES_PATH=$LAUNCHER_TEST_DIR/sources
mkdir -p $JOBSLOT_TEST_DIR
mkdir -p $LAUNCHER_TEST_DIR
mkdir -p $LAUNCHER_RUNCMD_PATH
mkdir -p $LAUNCHER_SOURCES_PATH


################################################
################ JOB SLOT TOOL #################
################################################
# Generate test script for Job Slot Tool
echo "Job Slot Tool test setup started."
echo "#!/bin/bash" > $JOBSLOT_TEST_DIR/run_jobslot_commands.sh

testcase=1
while [ $testcase -le $NUM_TESTCASES ]
do
	testcase_id=$testcase
	# make 2-digit testcase_id
	while [[ ${#testcase_id} -lt 2 ]] ; do
		testcase_id="0${testcase_id}"
	done
	json_file=$(find $JSON_PATH -iname "t${testcase_id}_*")
	echo "echo \"\""  >> $JOBSLOT_TEST_DIR/run_jobslot_commands.sh
	echo "echo \"################################################################################\"" >> $JOBSLOT_TEST_DIR/run_jobslot_commands.sh
	echo "echo \"$JOBSLOT_EXE_PATH $json_file\"" >> $JOBSLOT_TEST_DIR/run_jobslot_commands.sh
	echo "(sleep 2; echo -ne '\n') | $JOBSLOT_EXE_PATH $json_file" >> $JOBSLOT_TEST_DIR/run_jobslot_commands.sh
	echo "echo \"################################################################################\"" >> $JOBSLOT_TEST_DIR/run_jobslot_commands.sh
	echo "echo \"\""  >> $JOBSLOT_TEST_DIR/run_jobslot_commands.sh
	true $(( testcase++ ))
done

echo "Job Slot Tool test setup completed."
echo ""

################################################
################ LAUNCHER APP ##################
################################################
cd $LAUNCHER_TEST_DIR
echo "Launcher App test setup started."
echo "#!/bin/bash" > $LAUNCHER_TEST_DIR/run_launcher_commands.sh

# Generate Sources for Launcher App
# declaring array list and index iterator
declare -a run_params=()
testcase=1
# reading file in row mode, insert each line into array
while read line;
do
	run_params[$testcase]="$line"
	# echo ${run_params[$testcase]}
    let "testcase++"
done < $RUN_CMDS_PATH

testcase=1
while [ $testcase -le $NUM_TESTCASES ]
do
	input_sources=1
	testcase_id=$testcase
	# make 2-digit testcase_id
	while [[ ${#testcase_id} -lt 2 ]] ; do
		testcase_id="0${testcase_id}"
	done
	# Generate input sources files
	if test -f $LAUNCHER_SOURCES_PATH/t${testcase_id}_sources.txt
	then
		rm $LAUNCHER_SOURCES_PATH/t${testcase_id}_sources.txt
	fi
	num_sources=$NUM_SOURCES
	# if [ $testcase == 12 ]
	# then
	# 	num_sources=64
	# fi
	# echo "num_sources: " $num_sources
	while [ $input_sources -le $num_sources ]
	do
		if [ $testcase -eq 12 ] # special handling for Video composing usecase (multi-input usecase)
		then
			echo "$INPUT_SOURCES_PATH/${INPUTS[$testcase]} $INPUT_SOURCES_PATH/${INPUTS[$testcase]} $INPUT_SOURCES_PATH/${INPUTS[$testcase]} $INPUT_SOURCES_PATH/${INPUTS[$testcase]}" >> $LAUNCHER_SOURCES_PATH/t${testcase_id}_sources.txt
		else
			echo "$INPUT_SOURCES_PATH/${INPUTS[$testcase]}" >> $LAUNCHER_SOURCES_PATH/t${testcase_id}_sources.txt
		fi
		true $(( input_sources++ ))
	done
	# Generate run cmd files
	json_file=$(find $JSON_PATH -iname "t${testcase_id}_*")
	echo "job_description = $json_file" > $LAUNCHER_RUNCMD_PATH/t${testcase_id}_run_params.txt
	echo "cmdline = ${run_params[$testcase]}" >> $LAUNCHER_RUNCMD_PATH/t${testcase_id}_run_params.txt
	# Generate test script for Launcher App
	echo "echo \"\""  >> $LAUNCHER_TEST_DIR/run_launcher_commands.sh
	echo "echo \"################################################################################\"" >> $LAUNCHER_TEST_DIR/run_launcher_commands.sh
	echo "echo -e \"$LAUNCHER_EXE_PATH $LAUNCHER_SOURCES_PATH/t${testcase_id}_sources.txt $LAUNCHER_RUNCMD_PATH/t${testcase_id}_run_params.txt --log_level 1\"" >> $LAUNCHER_TEST_DIR/run_launcher_commands.sh
	echo -e "$LAUNCHER_EXE_PATH $LAUNCHER_SOURCES_PATH/t${testcase_id}_sources.txt $LAUNCHER_RUNCMD_PATH/t${testcase_id}_run_params.txt --log_level 1" >> $LAUNCHER_TEST_DIR/run_launcher_commands.sh
	echo "echo \"################################################################################\"" >> $LAUNCHER_TEST_DIR/run_launcher_commands.sh
	echo "echo \"\""  >> $LAUNCHER_TEST_DIR/run_launcher_commands.sh
	true $(( testcase++ ))
done
cd ..

echo "Launcher App test setup completed."
